##############################################################
# an extended Frame that makes menu/tool-bar's automatically;
# use: 'from guimaker import GuiMaker', 'python guimaker.py'
##############################################################


from Tkinter import *

class GuiMaker(Frame):
    def __init__(self, master=None):
	Frame.__init__(self, master) 
        self.pack({'expand':'yes', 'fill':'both'})  # make frame stretchable
        self.master.minsize(1,1)                    # make root box resizeable
        self.start()                                # subclass: set menu/toolBar
        self.makeMenuBar()                          # here: build menu-bar
        self.makeWidgets()                          # subclass?: add middle-part
        self.makeToolBar()                          # here: build tool-bar


    ################################################################
    # make expandable menu bar at the top: set 'menuBar' below
    ################################################################

    menuBar = []    # default

    def makeMenuBar(self, helpButton=1):
        menubar = Frame(self, 
                         {'relief': 'raised', 'bd': 2,
                           Pack: {'side':'top', 'expand':'yes', 'fill':'x'}})
        pulldowns = []
        for entry in self.menuBar:
            mbutton = Menubutton(menubar,
                               {'text':       entry[0],
                                'underline':  entry[1],
                                 Pack: {'side': 'left', 'padx': '1m'}})
            menu = self.addMenuItems(mbutton, entry[2])
            mbutton['menu'] = menu
            pulldowns.append(mbutton)
 
        apply(menubar.tk_menuBar, tuple(pulldowns))
        if helpButton:
            Button(menubar, {'text':   'Help',
                             'cursor': 'gumby',
                             'relief': 'flat',
                             'command': self.help,
                              Pack: {'side': 'right', 'padx': '1m'}})


    def addMenuItems(self, mbutton, items):
        menu = Menu(mbutton)
        for item in items:                     # scan nested items list
            if item == 'separator':            # string: add separator
                menu.add_separator({})
            elif type(item) == type([]):       # list: disabled item list
                for num in item:
                    menu.entryconfig(num, {"state" : "disabled"})
            elif type(item[2]) != type([]):
                menu.add_command({'label':     item[0],       # command: 
                                  'underline': item[1],       # add command
                                  'command':   item[2]})      # cmd=callable
            else:
                submenu = self.addMenuItems(mbutton, item[2])     # sublist:
                menu.add_cascade({'label':     item[0],           # recur...
                                  'underline': item[1],           # cmd=list
                                  'menu':      submenu})          # add cascade
        return menu


    ################################################################
    # make expandable button bar at the bottom: set 'toolBar' below
    ################################################################

    toolBar = []     # default

    def makeToolBar(self):
        toolbar = Frame(self, 
                         {'cursor': 'hand2',
                          'relief': 'sunken', 'bd': 2,
                           Pack: {'side':'bottom', 'expand':'yes', 'fill':'x'}})
        for item in self.toolBar: 
            Button(toolbar, {'text':    item[0], 
                             'command': item[1], 
                              Pack:     item[2]} )
 

    ################################################################
    # default 'middle' part: over-ride below if needed
    ################################################################

    def makeWidgets(self):
        Label(self, {'text':   self.__class__.__name__,
                     'cursor': 'crosshair',
                      Pack:    {'padx': 50, 'pady':50,
                                'expand':'yes', 'fill':'both'} })


###########################################################
# test if this file run directly ('python guiapp.py');
###########################################################

if __name__ == '__main__':
    from guimixin import *                 # GuiMixin: help, quit
    class TestApp(GuiMixin, GuiMaker):
        def start(self):
            self.menuBar = [ 
                ('File', 0,  
                    [('Quit',  0, self.quit)]
                )]
            self.toolBar = [
                ('Quit', self.quit, {'side':'left'})
                ]
    TestApp().mainloop()
