#!/usr/local/bin/python

from Tkinter  import *                              # get widgets
from guimixin import GuiMixin                       # get quit, notdone
from guimaker import GuiMaker                       # menu/toolbar builder

class ShellGui(GuiMixin, GuiMaker):                 # a frame + maker + mixins
    def start(self):
        self.pack({'expand': 'yes', 'fill': 'both'})
        self.setMenuBar()
        self.setToolBar()
        self.master.title("Shell Tools Listbox")
        self.master.iconname("Shell Tools")

    def handleList(self, event):
        index = self.listbox.curselection()           # on listbox double-click
        label = self.listbox.get(index)               # fetch selection text
        self.runCommand(label)                        # and call action here

    def makeWidgets(self):                            # add listbox in middle
        frame  = Frame(self)
        scroll = Scrollbar(frame)
        list   = Listbox(frame)
        frame.pack({'side':'top', 'expand':'yes', 'fill':'both'})
        list.config(
                 { 'yscrollcommand': scroll.set,
                   'relief': 'sunken',
                   'setgrid': 1,
                    Pack:{'side':'left', 'expand':'yes', 'fill':'both'} })
        scroll.config(
                 { 'command': list.yview,
                   'relief': 'sunken',
                    Pack:{'side':'right', 'fill':'both'} })
        pos = 0
        for (label, action) in self.fetchCommands():      # add to list-box
            list.insert(pos, label)                       # and menu/toolbars
            pos = pos + 1
        list.tk_listboxSingleSelect()
        list.bind('<Double-1>', self.handleList)          # set event handler
        self.listbox = list   

    def forToolBar(self, label):                          # put on toolbar?
        return 1                                          # default = all

    def setToolBar(self):
        self.toolBar = []
        for (label, action) in self.fetchCommands():
            if self.forToolBar(label):
                self.toolBar.append((label, action, {'side':'left'}))
        self.toolBar.append(('Quit', self.quit, {'side':'right'}))

    def setMenuBar(self):
        toolEntries  = []
        self.menuBar = [ 
            ('File',  0, [('Quit', -1, self.quit)]),    # pull-down name
            ('Tools', 0, toolEntries)                   # menu items list
            ]                                           # label,underline,action
        for (label, action) in self.fetchCommands():
            toolEntries.append((label, -1, action))      # add app items


####################################################
# delegate to type-specific subclasses again...
####################################################

class ListMenuGui(ShellGui):
    def fetchCommands(self):             # subclass: set 'myMenu'
        return self.myMenu 
    def runCommand(self, cmd):
        for (label, action) in self.myMenu: 
            if label == cmd: action()

class DictMenuGui(ShellGui):
    def fetchCommands(self):   return self.myMenu.items()
    def runCommand(self, cmd): self.myMenu[cmd]()

